"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request");
var channel_1 = require("./channel");
var logger_1 = require("./logger");
var Telegram = /** @class */ (function (_super) {
    __extends(Telegram, _super);
    function Telegram(bot) {
        return _super.call(this, { bot: bot, log: new logger_1.TelegramLogger(bot) }) || this;
    }
    Telegram.prototype.execute = function (body, parameters, handler) {
        var _this = this;
        var query = body['callback_query'];
        return new Promise(function (resolve) {
            _this.context.log.incoming(body);
            handler(_this.context.bot, (query || body), 'telegram', function (result) {
                // force to array. 
                result = result.length ? result : [result];
                _this.sendMessage(0, result, (query || body)).then(function (body) {
                    resolve(body);
                });
            }, function (err) { return resolve(err); });
        });
    };
    Telegram.prototype.sendMessage = function (index, messages, body) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var supportedMethods = [
                "message",
                "photo",
                "audio",
                "document",
                "location",
                "venue",
                "contact"
            ];
            var message = messages[index];
            var method = "sendMessage";
            if (message.type && supportedMethods.indexOf(message.type.toLowerCase()) >= 0) {
                method = message.type.toLowerCase();
                method = "send" + method.substr(0, 1).toUpperCase() + method.substr(1, method.length - 1);
            }
            var uri = "https://api.telegram.org/bot" + (process.env.RECIME_TELEGRAM_ACCESS_TOKEN || _this.context.bot.config.RECIME_TELEGRAM_ACCESS_TOKEN) + "/" + method;
            var options = {
                uri: uri,
                method: "POST",
                json: message.payload || message
            };
            options.json.chat_id = body.message.chat.id;
            request(options, function (err, response, result) {
                _this.context.log.outgoing(options, result);
                if (!err && response.statusCode === 200) {
                    if (index + 1 < messages.length) {
                        _this.sendMessage(++index, messages, body).then(resolve);
                    }
                    else {
                        return resolve();
                    }
                }
                else {
                    resolve({
                        message: {
                            text: JSON.stringify(result.error || response.body),
                            chat_id: options.json.chat_id
                        }
                    });
                }
            });
        });
    };
    return Telegram;
}(channel_1.Channel));
exports.Telegram = Telegram;
